unit ComPort;

interface

uses
  Classes, Windows, SysUtils;

type
  TComPort = class(TObject)
    FBaudRate : Integer;
    FName : string;
    FHandle : THandle;
    m_pBuf : Pointer;
    FFlags : DWORD;
    FReadTimeOut : DWORD;
    FWriteTimeOut : DWORD;
  private
    //function GetBinary() : Boolean;
    //procedure SetBinary(Value : Boolean);
    procedure SetReadTimeOut(Value: DWORD);
    procedure SetWriteTimeOut(Value: DWORD);
  public
    Constructor Create(Baud : Integer; Index : Integer);
    Destructor Destroy; override;
    function Open() : Boolean;
    procedure Close();
    function Read() : String;
    function Write(AData : string) : boolean;
    property ReadTimeOut : DWORD read FReadTimeOut write SetReadTimeOut;
    property WriteTimeOut : DWORD read FWriteTimeOut write SetWriteTimeOut;
    //property Binary : Boolean read GetBinary write SetBinary;
  end;

implementation 

Constructor TComPort.Create(Baud : Integer; Index : Integer);
begin
  FFlags := 1;
  FReadTimeOut := 300;
  FWriteTimeOut := 300;
  FBaudRate := Baud;
  FName := 'COM' + IntToStr(Index);
  m_pBuf := GetMemory(1024);
  inherited Create;
end;

Destructor TComPort.Destroy;
begin
  FreeMemory(m_pBuf);
  Inherited Destroy;
end;

procedure TComPort.SetReadTimeOut(Value: DWORD);
begin
  FReadTimeOut := Value;
end;

procedure TComPort.SetWriteTimeOut(Value: DWORD);
begin
  FWriteTimeOut := Value;
end;

{
function TComPort.GetBinary() : Boolean;
begin
  Result := (FFlags and $00000001) = $00000001;
end;

procedure TComPort.SetBinary(Value : Boolean);
begin
  if (Value) then
  begin
    FFlags := FFlags or ($00000001 shl 0);
  end else
  begin
    FFlags := FFlags and (not($00000001 shl 0));
  end;
end;
}

function TComPort.Open() : Boolean;
var
  PortDCB : TDCB;
  Timeouts : COMMTIMEOUTS;
begin
  Result := False;
  FHandle := CreateFile(PChar(FName),
                        GENERIC_READ or GENERIC_WRITE,
                        0, nil, OPEN_EXISTING,
			0, 0);

  if (FHandle = INVALID_HANDLE_VALUE) then
    exit;

  FillChar(PortDCB, sizeof(TDCB), #0);
  PortDCB.DCBlength := sizeof(TDCB);
  GetCommState(FHandle, PortDCB);

  PortDCB.BaudRate := FBaudRate;
  PortDCB.ByteSize := 8;
  PortDCB.Parity := NOPARITY;
  PortDCB.StopBits := ONESTOPBIT;
  PortDCB.Flags := FFlags;//4243;

  if (not SetCommState(FHandle, PortDCB)) then
    exit;

  Timeouts.ReadIntervalTimeout := MAXDWORD;
  Timeouts.ReadTotalTimeoutMultiplier := MAXDWORD;
  Timeouts.ReadTotalTimeoutConstant := FReadTimeOut;
  Timeouts.WriteTotalTimeoutMultiplier := MAXDWORD;
  Timeouts.WriteTotalTimeoutConstant := FWriteTimeOut;

  if (not SetCommTimeouts(FHandle, Timeouts)) then
    exit;

  EscapeCommFunction(FHandle, 11);

  Result := True;
end;

procedure TComPort.Close();
begin
  CloseHandle(FHandle);
  FHandle := INVALID_HANDLE_VALUE;
end;

function TComPort.Read() : String;
var
  dwRead : Cardinal;
begin
  Result := '';
  ReadFile(FHandle, m_pBuf^, 1024, dwRead, nil);
  Result := String(m_pBuf);
  SetLength(Result, dwRead);
end;

Function TComPort.Write(AData : string) : boolean;
var
  m_ppBuf : Pointer;
  dwWrote, dwCurWrote : Cardinal;
begin
  Result := False;
  m_ppBuf := @Pointer(AData)^;
  dwWrote := 0;
  while (dwWrote < Length(AData)) do
  begin
    if (not WriteFile(FHandle, Pointer(LongInt(m_ppBuf) + dwWrote)^, length(AData) - dwWrote, dwCurWrote, nil)) then
    begin
      exit;
    end;
      
    Inc(dwWrote, dwCurWrote);
  end;

  Result := True;
end;

end.
